<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Models\AccountInfo;
use App\Models\drivewayAccount;
use Illuminate\Http\Request;
use Symfony\Component\HttpFoundation\Response;
use Yajra\DataTables\Facades\DataTables;
use Gate;

class DrivewayAccountController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        abort_if(Gate::denies('driveway_account_access'), Response::HTTP_FORBIDDEN, '403 Forbidden');
        if($request->ajax())
        {

            $driveway_account = drivewayAccount::where('branch_id',session()->get('branch_id'))->get();

            if ($request->startDate != null && $request->endDate) {
                $req_start_date = strtr($request->startDate,'/','-');
                $start_date = date('Y-m-d',strtotime($req_start_date));
                $req_end_date = strtr($request->endDate,'/','-');
                $end_date = date('Y-m-d',strtotime($req_end_date));

                $driveway_account = drivewayAccount::where('branch_id',session()->get('branch_id'))->wherebetween('date',[$start_date,$end_date])->get();
            }

                $table = Datatables::of($driveway_account);
                $table->addIndexColumn();

                $table->addColumn('placeholder', '&nbsp;');
                $table->addColumn('actions', '&nbsp;');

                $table->editColumn('actions', function ($row) {
                    $viewGate = 'driveway_account_show';
                    $editGate = 'driveway_account_edit';
                    $deleteGate = 'driveway_account_delete';
                    $crudRoutePart = 'driveway-accounts';

                    return view('partials.datatablesActions', compact(
                    'viewGate',
                    'editGate',
                    'deleteGate',
                    'crudRoutePart',
                    'row'
                    ));
                });


                $table->editColumn('date', function ($row) {
                    return $row->date ? date('d/m/Y',strtotime($row->date)) : '';
                });

                $table->editColumn('account_name',function($row){
                    return $row->account_info ? $row->account_info->title : '';
                });

                $table->editColumn('amount', function ($row) {
                    return $row->amount ? $row->amount : '';
                });

                $table->rawColumns(['actions', 'placeholder', 'Account']);

                return $table->make(true);
            }

            return view('admin.drivewayAccount.index');
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        abort_if(Gate::denies('driveway_account_create'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $account_info = AccountInfo::all()->pluck('title','id')->prepend(trans('global.pleaseSelect'),'');
        return view('admin.drivewayAccount.create',compact('account_info'));
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        abort_if(Gate::denies('driveway_account_create'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $request->validate([
            'date' => 'required',
            'invoice_no.*' => 'required|unique:driveway_accounts,invoice_no',
            'account_name.*' => 'required',
            'amount.*' => 'required|numeric',
            'remarks.*' => 'nullable',
        ],[
            'invoice_no.*.required' => 'The invoice no field is required.',
            'invoice_no.*.unique' => 'The invoice no. has already been taken.',
            'account_name.*.required' => 'The account name is required.',
            'amount.*.required' => 'The amount field is required',
            'amount.*.numeric' => 'The amount must be number',
        ]);

        $req_date = strtr($request->date,'/','-');
        $date = date('Y-m-d',strtotime($req_date));
        foreach ($request->invoice_no as $key => $value) {
            drivewayAccount::create([
                'date' => $date,
                'invoice_no' => $request->invoice_no[$key],
                'account_info_id' => $request->account_name[$key],
                'branch_id' => session()->get('branch_id'),
                'remarks' => $request->remarks[$key],
                'amount' => $request->amount[$key],
            ]);
        }

        if ($request->ajax()) {
            return response()->json(['success'=>true,'message'=>'Account is created successfully!','url'=> route('admin.driveway-accounts.index')]);
        }
        else{
            toastr()->success("Account is created successfully!");
            return redirect()->route('admin.driveway-accounts.index');
        }

    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        abort_if(Gate::denies('driveway_account_show'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $account = drivewayAccount::find($id);
        return view('admin.drivewayAccount.show',compact('account'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        abort_if(Gate::denies('driveway_account_edit'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $account = drivewayAccount::find($id);
        $account_info = AccountInfo::all()->pluck('title','id')->prepend(trans('global.pleaseSelect'),'');
        return view('admin.drivewayAccount.edit',compact('account_info','account'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request,$id)
    {
        abort_if(Gate::denies('driveway_account_edit'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $account = drivewayAccount::find($id);
        $request->validate([
            'date' => 'required',
            'invoice_no' => 'required',
            'account_name.*' => 'required',
            'amount' => 'required|numeric',
            'remarks' => 'nullable',
        ]);

        $req_date = strtr($request->date,'/','-');
        $date = date('Y-m-d',strtotime($req_date));

        $account->update([
            'date' => $date,
            'invoice_no' => $request->invoice_no,
            'account_info_id' => $request->account_name,
            'branch_id' => session()->get('branch_id'),
            'remarks' => $request->remarks,
            'amount' => $request->amount,
        ]);
        toastr()->info('Account has been updated successfully!');
        return redirect()->route('admin.driveway-accounts.index');
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        abort_if(Gate::denies('driveway_account_delete'), Response::HTTP_FORBIDDEN, '403 Forbidden');

        $account = drivewayAccount::find($id);
        if (!empty($account)) {
            $account->forceDelete();
            toastr()->error('Account has been deleted!');

            return back();
        }
    }

    public function massDestroy(Request $request)
    {
        abort_if(Gate::denies('driveway_account_delete'), Response::HTTP_FORBIDDEN, '403 Forbidden');
        $request->validate([
            'ids'   => 'required|array',
            'ids.*' => 'exists:driveway_accounts,id',
        ]);

        foreach($request->ids as $id){
            $driveway_account = drivewayAccount::find($id);
            $driveway_account->forceDelete();
        }
        toastr()->error('Account has been deleted!');
        return response(null, Response::HTTP_NO_CONTENT);
    }
}
